<?php

namespace Drupal\url_restriction_by_role\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;
use Drupal\path_alias\AliasManagerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\RequestEvent;

/**
 * Event Subscriber for Url Restrictions.
 */
class UrlRestrictionByRole implements EventSubscriberInterface {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'url_restriction_by_role.settings';


  /**
   * Error Message.
   *
   * @var string
   */
  const ERROR_MESSAGE = 'error_message';

  /**
 * Use Custom Error Message.
 *
 * @var string
 */
  const USE_CUSTOM_ERROR_MESSAGE = 'use_custom_error_message';

  /**
   * Form table name.
   *
   * @var string
   */
  const FORM_TABLE = 'urls';

  /**
   * Column url.
   *
   * @var string
   */
  const FORM_COLUMN_URL = 'url';

  /**
   * Column enabled.
   *
   * @var string
   */
  const FORM_COLUMN_ENABLED = 'enabled';

  /**
   * Column role.
   *
   * @var string
   */
  const FORM_COLUMN_ROLE = 'role';

  /**
   * Configuration object.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * User object.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The path matcher.
   *
   * @var \Drupal\Core\Path\PathMatcher
   */
  protected $pathMatcher;

  /**
   * The current path.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * The path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;


  /**
   * Constructor.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config
   *   The config service.
   * @param \Drupal\Core\Path\PathMatcherInterface $path_matcher
   *   The path matcher.
   * @param \Drupal\Core\Path\CurrentPathStack $current_path
   *   The current path.
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   The path alias manager.

   */
  public function __construct(
    AccountProxyInterface $currentUser,
    ConfigFactoryInterface $config,
    PathMatcherInterface $path_matcher,
    CurrentPathStack $current_path,
    AliasManagerInterface $alias_manager,

  ) {
    $this->currentUser = $currentUser;
    $this->config = $config;
    $this->pathMatcher = $path_matcher;
    $this->currentPath = $current_path;
    $this->aliasManager = $alias_manager;
  }

  /**
   * Restrict access to URLs defined.
   */
  public function onRequest(RequestEvent $event) {
    $error_message = $this->config->get(static::SETTINGS)->get(static::ERROR_MESSAGE);
    $use_custom_error_message = $this->config->get(static::SETTINGS)->get(static::USE_CUSTOM_ERROR_MESSAGE);
    $roles = $this->currentUser->getRoles();
    $basic_site_information_page_403 = $this->config->get('system.site')->get('page.403');
    //Get event info
    $request = $event->getRequest();
    $current_path = $this->currentPath->getPath($request);
    $alias =  $this->aliasManager->getAliasByPath($current_path);

    //get all configured paths
    $configured_paths = $this->config->get(static::SETTINGS)->get('urls');
    if($configured_paths){
      $configured_paths = array_keys($configured_paths);
      foreach($configured_paths as $check){
        //we check for path but also for alias, this includes wildcards
          if ($this->pathMatcher->matchPath($current_path, $check ) || $this->pathMatcher->matchPath($alias, $check )) {
            $options = $this->config->get(static::SETTINGS)->get(static::FORM_TABLE . '.' . $check);
            if (!empty($options) && $options[static::FORM_COLUMN_ENABLED]) {
              $role = $options[static::FORM_COLUMN_ROLE];
              if ((is_string($role) && in_array($role, $roles)) || (is_array($role) && count(array_intersect($role, $roles)) == 0)) {
                if (!$use_custom_error_message) {
                  if (empty($basic_site_information_page_403)) {
                    $response = new RedirectResponse('/system/403');
                    $event->setResponse($response);
                  }
                  else {
                    $response = new RedirectResponse($basic_site_information_page_403);
                    $event->setResponse($response);
                  }
                }
                else {
                  $response = new Response(empty($error_message) ? 'You do not have access to this page' : $error_message, 403);
                  $event->setResponse($response);
                }
        
              }
            }
  
          }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[KernelEvents::REQUEST][] = ['onRequest'];
    return $events;
  }

}